<?php
/**
 * Settings class file.
 *
 * @package MNMLD Themes/Settings
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Settings class.
 */
class MNMLD_Themes_Settings {

	/**
	 * The single instance of MNMLD_Themes_Settings.
	 *
	 * @var     object
	 * @access  private
	 * @since   1.0.0
	 */
	private static $_instance = null; 

	/**
	 * The main plugin object.
	 *
	 * @var     object
	 * @access  public
	 * @since   1.0.0
	 */
	public $parent = null;

	/**
	 * Prefix for plugin settings.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $base = '';

	/**
	 * Available settings for plugin.
	 *
	 * @var     array
	 * @access  public
	 * @since   1.0.0
	 */
	public $settings = array();

	/**
	 * Constructor function.
	 *
	 * @param object $parent Parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;

		$this->base = 'MNMLD_Themes_';

		// Initialise settings.
		add_action( 'init', array( $this, 'init_settings' ), 11 );

		// Register plugin settings.
		add_action( 'admin_init', array( $this, 'register_settings' ) );

		// Add settings page to menu.
		add_action( 'admin_menu', array( $this, 'add_menu_item' ) );

		// Add settings link to plugins page.
		add_filter(
			'plugin_action_links_' . plugin_basename( $this->parent->file ),
			array(
				$this,
				'add_settings_link',
			)
		);

		// Configure placement of plugin settings page. See readme for implementation.
		add_filter( $this->base . 'menu_settings', array( $this, 'configure_settings' ) );
	}

	/**
	 * Initialise settings
	 *
	 * @return void
	 */
	public function init_settings() {
		$this->settings = $this->settings_fields();
	}

	/**
	 * Add settings page to admin menu
	 *
	 * @return void
	 */
	public function add_menu_item() {

		$args = $this->menu_settings();

		// Do nothing if wrong location key is set.
		if ( is_array( $args ) && isset( $args['location'] ) && function_exists( 'add_' . $args['location'] . '_page' ) ) {
			switch ( $args['location'] ) {
				case 'options':
				case 'submenu':
					$page = add_submenu_page( $args['parent_slug'], $args['page_title'], $args['menu_title'], $args['capability'], $args['menu_slug'], $args['function'] );
					break;
				case 'menu':
					$page = add_menu_page( $args['page_title'], $args['menu_title'], $args['capability'], $args['menu_slug'], $args['function'], $args['icon_url'], $args['position'] );
					break;
				default:
					return;
			}
			add_action( 'admin_print_styles-' . $page, array( $this, 'settings_assets' ) );
		}
	}

	/**
	 * Prepare default settings page arguments
	 *
	 * @return mixed|void
	 */
	private function menu_settings() {
		return apply_filters(
			$this->base . 'menu_settings',
			array(
				'location'    => 'menu', // Possible settings: options, menu, submenu.
				'parent_slug' => 'themes.php',
				'page_title'  => __( 'MNMLD Themes - Settings', 'mnmld-themes' ),
				'menu_title'  => __( 'MNMLD', 'mnmld-themes' ),
				'capability'  => 'manage_options',
				'menu_slug'   => $this->parent->_token . '_settings',
				'function'    => array( $this, 'settings_page' ),
				'icon_url'    => plugins_url( '../assets/images/admin.png' , __FILE__ ),
				'position'    => 3,
			)
		);
	}

	/**
	 * Container for settings page arguments
	 *
	 * @param array $settings Settings array.
	 *
	 * @return array
	 */
	public function configure_settings( $settings = array() ) {
		return $settings;
	}

	/**
	 * Load settings JS & CSS
	 *
	 * @return void
	 */
	public function settings_assets() {

		// We're including the farbtastic script & styles here because they're needed for the colour picker
		// If you're not including a colour picker field then you can leave these calls out as well as the farbtastic dependency for the wpt-admin-js script below.
		wp_enqueue_style( 'farbtastic' );
		wp_enqueue_script( 'farbtastic' );

		// We're including the WP media scripts here because they're needed for the image upload field.
		// If you're not including an image upload then you can leave this function call out.
		wp_enqueue_media();

		wp_register_script( $this->parent->_token . '-settings-js', $this->parent->assets_url . 'js/settings.js', array( 'farbtastic', 'jquery' ), '1.0.0', true );
		wp_enqueue_script( $this->parent->_token . '-settings-js' );
	}

	/**
	 * Add settings link to plugin list table
	 *
	 * @param  array $links Existing links.
	 * @return array        Modified links.
	 */
	public function add_settings_link( $links ) {
		$settings_link = '<a href="options-general.php?page=' . $this->parent->_token . '_settings">' . __( 'Settings', 'mnmld-themes' ) . '</a>';
		array_push( $links, $settings_link );
		return $links;
	}

	/**
	 * Build settings fields
	 *
	 * @return array Fields to be displayed on settings page
	 */
	private function settings_fields() {
	
	
		//Analytics
		$settings['Analytics'] = array(
			'title'       => __( 'Analytics Code', 'mnmld-themes' ),
			'description' => '',
			'fields'      => array(
				array(
					'id'          => 'analytics_toggle',
					'label'       => __( 'Goolge Analytics/Stats Code', 'mnmld-themes' ),
					'description' => __( 'Analytics code will disable tracking when admin/editor is logged in.', 'mnmld-themes' ),
					'type'        => 'radio',
					'options'     => array(
						'on' => 'Enabled',
						'off'   => 'Disabled',
					),
					'default'     => 'off',
				),
				array(
					'id'          => 'analytics_code',
					'label'       => __( 'Google Analytics/Stats Code', 'mnmld-themes' ),
					'description' => '',
					'type'        => 'textarea',
					'default'     => '',
					'placeholder' => __( 'Paste Code Here', 'mnmld-themes' ),
				),
				
		));
	
		
		//Contact Form
		$settings['Contact'] = array(
			'title'       => __( 'Contact Page', 'mnmld-themes' ),
			'description' => '',
			'fields'      => array(
				array(
					'id'          => 'contact_shortcode',
					'label'       => __( 'Contact Form Shortcode', 'mnmld-themes' ),
					'description' => __( '<b>[mnmld-contact]</b><br />Use shortcode in your pages to display the form', 'mnmld-themes' ),
					'type'        => 'custom',
				),
				array(
					'id'          => 'contact_success',
					'label'       => __( 'Success Message', 'mnmld-themes' ),
					'description' => __( 'A message appears after visitor sends the message.', 'mnmld-themes' ),
					'type'        => 'textarea',
					'default'     => __( 'Your message has been sent! <br />Thank you for your message.', 'mnmld-themes' ),
					'placeholder' => '',
				),
				array(
					'id'          => 'contact_map_shortcode',
					'label'       => __( 'Google Maps Shortcode', 'mnmld-themes' ),
					'description' => __( '<b>[mnmld-map]</b><br />Use shortcode in your pages to display the map', 'mnmld-themes' ),
					'type'        => 'custom',
				),
				array(
					'id'          => 'contact_map_code',
					'label'       => __( 'Google Maps Code', 'mnmld-themes' ),
					'description' => __( 'Get Maps Code', 'mnmld-themes' ) . ' <a href="https://www.maps.ie/create-google-map/" target="_blank">Map Code Generator</a>',
					'type'        => 'textarea',
					'placeholder' => '',
				),
				
		));
		
		//User Profiles
		$settings['User_Links'] = array(
			'title'       => __( 'User Social Links', 'mnmld-themes' ),
			'description' => __( 'Adding social networks links in user page.', 'mnmld-themes' ),
			'fields'      => array(
				array(
					'id'          => 'user_links',
					'label'       => __( 'User Social Links', 'mnmld-themes' ),
					'description' => '',
					'type'        => 'radio',
					'options'     => array(
						'on' => 'Enabled',
						'off'   => 'Disabled',
					),
					'default'     => 'on',
				),
				
		));
		
		//Contact Form
		$settings['Performance'] = array(
			'title'       => __( 'Performance', 'mnmld-themes' ),
			'description' => __( 'Performance Hooks', 'mnmld-themes' ),
			'fields'      => array(
				
				array(
					'id'          => 'remove_querystring',
					'label'       => __( 'Remove Query Strings', 'mnmld-themes' ),
					'description' => __( 'Remove query strings from all static resources', 'mnmld-themes' ),
					'type'        => 'checkbox',
					'default'     => 'on',
				),
				
				array(
					'id'          => 'remove_generator',
					'label'       => __( 'Remove WP Generator', 'mnmld-themes' ),
					'description' => __( 'Removes wordpress version meta tag for more security.', 'mnmld-themes' ),
					'type'        => 'checkbox',
					'default'     => '',
				),
				
				array(
					'id'          => 'emoji_control',
					'label'       => __( 'Disable Emojis', 'mnmld-themes' ),
					'description' => __( 'Disables Emojis extra scripts', 'mnmld-themes' ),
					'type'        => 'checkbox',
					'default'     => '',
				),
				
				array(
					'id'          => 'fse_css',
					'label'       => __( 'Remove Full Site Editing Inline CSS', 'mnmld-themes' ),
					'description' => __( 'USE WITH CAUTION: Remoces unecessary inline css produced by wordpress FSE functions', 'mnmld-themes' ),
					'type'        => 'checkbox',
					'default'     => '',
				),
				
		));
		
		//User Profiles
		$settings['Misc'] = array(
			'title'       => __( 'Misc', 'mnmld-themes' ),
			'description' => '',
			'fields'      => array(
				array(
					'id'          => 'login_logo',
					'label'       => __( 'Login Logo', 'mnmld-themes' ),
					'description' => __( 'Recommended Image height: 90px', 'mnmld-themes' ),
					'type'        => 'image',
					'default'     => '',
					'placeholder' => '',
				),
				
		));
		
		$settings = apply_filters( $this->parent->_token . '_settings_fields', $settings );

		return $settings;
	}

	/**
	 * Register plugin settings
	 *
	 * @return void
	 */
	public function register_settings() {
		if ( is_array( $this->settings ) ) {

			// Check posted/selected tab.
			//phpcs:disable
			$current_section = '';
			if ( isset( $_POST['tab'] ) && $_POST['tab'] ) {
				$current_section = $_POST['tab'];
			} else {
				if ( isset( $_GET['tab'] ) && $_GET['tab'] ) {
					$current_section = $_GET['tab'];
				}
			}
			//phpcs:enable

			foreach ( $this->settings as $section => $data ) {

				if ( $current_section && $current_section !== $section ) {
					continue;
				}

				// Add section to page.
				add_settings_section( $section, $data['title'], array( $this, 'settings_section' ), $this->parent->_token . '_settings' );

				foreach ( $data['fields'] as $field ) {

					// Validation callback for field.
					$validation = '';
					if ( isset( $field['callback'] ) ) {
						$validation = $field['callback'];
					}

					// Register field.
					$option_name = $this->base . $field['id'];
					register_setting( $this->parent->_token . '_settings', $option_name, $validation );

					// Add field to page.
					add_settings_field(
						$field['id'],
						$field['label'],
						array( $this->parent->admin, 'display_field' ),
						$this->parent->_token . '_settings',
						$section,
						array(
							'field'  => $field,
							'prefix' => $this->base,
						)
					);
				}

				if ( ! $current_section ) {
					break;
				}
			}
		}
	}

	/**
	 * Settings section.
	 *
	 * @param array $section Array of section ids.
	 * @return void
	 */
	public function settings_section( $section ) {
		$html = '<p> ' . $this->settings[ $section['id'] ]['description'] . '</p>' . "\n";
		echo $html; 
	}

	/**
	 * Load settings page content.
	 *
	 * @return void
	 */
	public function settings_page() {

		// Build page HTML.
		$html      = '<div class="wrap" id="' . $this->parent->_token . '_settings">' . "\n";
			$html .= '<h2>' . __( 'MNMLD Themes Options', 'mnmld-themes' ) . '</h2>' . "\n";

			$tab = '';
		//phpcs:disable
		if ( isset( $_GET['tab'] ) && $_GET['tab'] ) {
			$tab .= $_GET['tab'];
		}
		//phpcs:enable

		// Show page tabs.
		if ( is_array( $this->settings ) && 1 < count( $this->settings ) ) {

			$html .= '<h2 class="nav-tab-wrapper">' . "\n";

			$c = 0;
			foreach ( $this->settings as $section => $data ) {

				// Set tab class.
				$class = 'nav-tab';
				if ( ! isset( $_GET['tab'] ) ) { 
					if ( 0 === $c ) {
						$class .= ' nav-tab-active';
					}
				} else {
					if ( isset( $_GET['tab'] ) && $section == $_GET['tab'] ) { 
						$class .= ' nav-tab-active';
					}
				}

				// Set tab link.
				$tab_link = add_query_arg( array( 'tab' => $section ) );
				if ( isset( $_GET['settings-updated'] ) ) { 
					$tab_link = remove_query_arg( 'settings-updated', $tab_link );
				}

				// Output tab.
				$html .= '<a href="' . $tab_link . '" class="' . esc_attr( $class ) . '">' . esc_html( $data['title'] ) . '</a>' . "\n";

				++$c;
			}

			$html .= '</h2>' . "\n";
		}

			$html .= '<form method="post" action="options.php" enctype="multipart/form-data">' . "\n";

				// Get settings fields.
				ob_start();
				settings_fields( $this->parent->_token . '_settings' );
				do_settings_sections( $this->parent->_token . '_settings' );
				$html .= ob_get_clean();

				$html     .= '<p class="submit">' . "\n";
					$html .= '<input type="hidden" name="tab" value="' . esc_attr( $tab ) . '" />' . "\n";
					$html .= '<input name="Submit" type="submit" class="button-primary" value="' . esc_attr( __( 'Save Settings', 'mnmld-themes' ) ) . '" />' . "\n";
				$html     .= '</p>' . "\n";
			$html         .= '</form>' . "\n";
		$html             .= '</div>' . "\n";

		echo $html; 
	}

	/**
	 * Main MNMLD_Themes_Settings Instance
	 *
	 * Ensures only one instance of MNMLD_Themes_Settings is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @see MNMLD_Themes()
	 * @param object $parent Object instance.
	 * @return object MNMLD_Themes_Settings instance
	 */
	public static function instance( $parent ) {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self( $parent );
		}
		return self::$_instance;
	} // End instance()

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, esc_html( __( 'Cloning of MNMLD_Themes_API is forbidden.' ) ), esc_attr( $this->parent->_version ) );
	} // End __clone()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, esc_html( __( 'Unserializing instances of MNMLD_Themes_API is forbidden.' ) ), esc_attr( $this->parent->_version ) );
	} // End __wakeup()

}
