<?php
/**
 * Main plugin class file.
 *
 * @package MNMLD Themes/Includes
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Main plugin class.
 */
class MNMLD_Themes {

	/**
	 * The single instance of MNMLD_Themes.
	 *
	 * @var     object
	 * @access  private
	 * @since   1.0.0
	 */
	private static $_instance = null; 

	/**
	 * Local instance of MNMLD_Themes_Admin_API
	 *
	 * @var MNMLD_Themes_Admin_API|null
	 */
	public $admin = null;

	/**
	 * Settings class object
	 *
	 * @var     object
	 * @access  public
	 * @since   1.0.0
	 */
	public $settings = null;

	/**
	 * The version number.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $_version; 

	/**
	 * The token.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $_token; 

	/**
	 * The main plugin file.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $file;

	/**
	 * The main plugin directory.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $dir;

	/**
	 * The plugin assets directory.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $assets_dir;

	/**
	 * The plugin assets URL.
	 *
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $assets_url;

	/**
	 * Constructor funtion.
	 *
	 * @param string $file File constructor.
	 * @param string $version Plugin version.
	 */
	public function __construct( $file = '', $version = '1.0.0' ) {
		$this->_version = $version;
		$this->_token   = 'MNMLD_Themes';

		// Load plugin environment variables.
		$this->file       = $file;
		$this->dir        = dirname( $this->file );
		$this->assets_dir = trailingslashit( $this->dir ) . 'assets';
		$this->assets_url = esc_url( trailingslashit( plugins_url( '/assets/', $this->file ) ) );

		register_activation_hook( $this->file, array( $this, 'install' ) );
		
		if( get_option( 'mnmld_themes_user_links', 'on' ) == 'on'){
			// Additional contact links
			add_filter( 'user_contactmethods', array( $this, 'user_contact_links' ), 10, 1 );
		}
		
		// Load frontend JS & CSS.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ), 20 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 10 );

		// Load admin JS & CSS.
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ), 10, 1 );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_styles' ), 10, 1 );

		// Load API for generic admin functions.
		if ( is_admin() ) {
			$this->admin = new MNMLD_Themes_Admin_API();
		}

		// Handle localisation.
		$this->load_plugin_textdomain();
		add_action( 'init', array( $this, 'load_localisation' ), 0 );
	} // End __construct ()


	
	/**
	 * Load frontend CSS.
	 *
	 * @access  public
	 * @return void
	 * @since   1.0.0
	 */
	public function enqueue_styles() {
		wp_register_style( 'mnmld-themes', esc_url( $this->assets_url ) . 'css/mnmld-themes.css', array(), $this->_version );
		wp_enqueue_style( 'mnmld-themes' );
	} // End enqueue_styles ()

	
	/**
	 * Load frontend JS.
	 *
	 * @access  public
	 * @return void
	 * @since   1.0.0
	 */
	public function enqueue_scripts() {
		wp_register_script( 'mnmld-themes', esc_url( $this->assets_url ) . 'js/mnmld-themes.js', array(), $this->_version );
		wp_enqueue_script( 'mnmld-themes' );
	} // End enqueue_scripts ()


	/**
	 * Admin enqueue style.
	 *
	 * @param string $hook Hook parameter.
	 *
	 * @return void
	 */
	public function admin_enqueue_styles( $hook = '' ) {
		wp_register_style( $this->_token . '-admin', esc_url( $this->assets_url ) . 'css/admin.css', array(), $this->_version );
		wp_enqueue_style( $this->_token . '-admin' );
	} // End admin_enqueue_styles ()

	/**
	 * Load admin Javascript.
	 *
	 * @access  public
	 *
	 * @param string $hook Hook parameter.
	 *
	 * @return  void
	 * @since   1.0.0
	 */
	public function admin_enqueue_scripts( $hook = '' ) {
		wp_register_script( $this->_token . '-admin', esc_url( $this->assets_url ) . 'js/admin.js', array( 'jquery' ), $this->_version, true );
		wp_enqueue_script( $this->_token . '-admin' );
	} // End admin_enqueue_scripts ()

	/**
	 * Load plugin localisation
	 *
	 * @access  public
	 * @return  void
	 * @since   1.0.0
	 */
	public function load_localisation() {
		load_plugin_textdomain( 'mnmld-themes', false, dirname( plugin_basename( $this->file ) ) . '/lang/' );
	} // End load_localisation ()

	/**
	 * Load plugin textdomain
	 *
	 * @access  public
	 * @return  void
	 * @since   1.0.0
	 */
	public function load_plugin_textdomain() {
		$domain = 'mnmld-themes';

		$locale = apply_filters( 'plugin_locale', get_locale(), $domain );

		load_textdomain( $domain, WP_LANG_DIR . '/' . $domain . '/' . $domain . '-' . $locale . '.mo' );
		load_plugin_textdomain( $domain, false, dirname( plugin_basename( $this->file ) ) . '/lang/' );
	} // End load_plugin_textdomain ()

	/**
	 * Main MNMLD_Themes Instance
	 *
	 * Ensures only one instance of MNMLD_Themes is loaded or can be loaded.
	 *
	 * @param string $file File instance.
	 * @param string $version Version parameter.
	 *
	 * @return Object MNMLD_Themes instance
	 * @see MNMLD_Themes()
	 * @since 1.0.0
	 * @static
	 */
	public static function instance( $file = '', $version = '1.0.0' ) {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self( $file, $version );
		}

		return self::$_instance;
	} // End instance ()

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, esc_html( __( 'Cloning of MNMLD_Themes is forbidden' ) ), esc_attr( $this->_version ) );

	} // End __clone ()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, esc_html( __( 'Unserializing instances of MNMLD_Themes is forbidden' ) ), esc_attr( $this->_version ) );
	} // End __wakeup ()

	/**
	 * Installation. Runs on activation.
	 *
	 * @access  public
	 * @return  void
	 * @since   1.0.0
	 */
	public function install() {
		$this->_log_version_number();
	} // End install ()

	/**
	 * Log the plugin version number.
	 *
	 * @access  public
	 * @return  void
	 * @since   1.0.0
	 */
	private function _log_version_number() { 
		update_option( $this->_token . '_version', $this->_version );
	} // End _log_version_number ()
	
	
	
	//Extending user contact methods
	public function user_contact_links( $links ) {
		
		// Define MNMLD theme prefix
		$mnmld_theme_prefix = __MNMLD_THEME_PREFIX; 
		
		if( $mnmld_theme_prefix == 'devnode' ) {
			
			$links['twitter']    = __( 'Twitter', 'mnmld-themes' );
			$links['youtube']    = __( 'YouTube', 'mnmld-themes' );
			$links['github']     = __( 'GitHub', 'mnmld-themes' );
			$links['tiktok']     = __( 'Tiktok', 'mnmld-themes' );
			$links['instagram']  = __( 'Instagram', 'mnmld-themes' );
			$links['pinterest']  = __( 'Pinterest', 'mnmld-themes' );
			$links['facebook']   = __( 'Facebook', 'mnmld-themes' );
			$links['vimeo']      = __( 'Vimeo', 'mnmld-themes' );
		}else{
			$links['facebook']   = __( 'Facebook', 'mnmld-themes' );
			$links['instagram']  = __( 'Instagram', 'mnmld-themes' );
			$links['twitter']    = __( 'Twitter', 'mnmld-themes' );
			$links['pinterest']  = __( 'Pinterest', 'mnmld-themes' );
			$links['youtube']    = __( 'YouTube', 'mnmld-themes' );
			$links['vimeo']      = __( 'Vimeo', 'mnmld-themes' );
			$links['tiktok']     = __( 'Tiktok', 'mnmld-themes' );
			$links['tumblr']     = __( 'Tumblr', 'mnmld-themes' );
		}
		
		return $links;
	}

}
